/**
 * ProjectLaogai
 *
 * Copyright 2019  <seil0@mosad.xyz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

package org.mosad.seil0.projectlaogai.fragments

import android.graphics.Typeface
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.afollestad.materialdialogs.MaterialDialog
import kotlinx.android.synthetic.main.fragment_home.*
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import org.mosad.seil0.projectlaogai.R
import org.mosad.seil0.projectlaogai.controller.CacheController.Companion.mensaCurrentWeek
import org.mosad.seil0.projectlaogai.controller.CacheController.Companion.timetables
import org.mosad.seil0.projectlaogai.hsoparser.DataTypes
import org.mosad.seil0.projectlaogai.hsoparser.Meal
import org.mosad.seil0.projectlaogai.hsoparser.NotRetardedCalendar
import org.mosad.seil0.projectlaogai.uicomponents.DayCardView
import org.mosad.seil0.projectlaogai.uicomponents.LessonLinearLayout
import org.mosad.seil0.projectlaogai.uicomponents.MealLinearLayout
import java.text.SimpleDateFormat
import java.util.*

/**
 * The "home" controller class
 * contains all needed parts to display the apps home screen
 */
class HomeFragment : Fragment() {

    val formatter = SimpleDateFormat("E dd.MM", Locale.GERMANY)

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        val view: View = inflater.inflate(R.layout.fragment_home, container, false)

        addMensaMenu()
        addTimeTable()

        // Inflate the layout for this fragment
        return view
    }

    /**
     * add the current mensa meal to the home screens
     */
    private fun addMensaMenu() {
        doAsync {

            var dayMeals: ArrayList<Meal>
            val cal = Calendar.getInstance()
            val mensaCardView = DayCardView(context!!)

            uiThread {

                if (cal.get(Calendar.HOUR_OF_DAY) < 15) {
                    dayMeals = mensaCurrentWeek.days[NotRetardedCalendar().getDayOfWeekIndex()].meals
                    mensaCardView.setDayHeading(resources.getString(R.string.today_date, formatter.format(cal.time)))
                } else {
                    dayMeals = mensaCurrentWeek.days[NotRetardedCalendar().getTomorrowWeekIndex()].meals
                    cal.add(Calendar.DATE, 1)
                    mensaCardView.setDayHeading(resources.getString(R.string.tomorrow_date, formatter.format(cal.time)))
                }

                if (dayMeals.size >= 2) {
                    // get the index of the first meal, not a "Schneller Teller"
                    loop@ for ((i, meal) in dayMeals.withIndex()) {
                        if (meal.heading.contains("Essen")) {

                            val meal1Layout = MealLinearLayout(context)
                            meal1Layout.setMeal(dayMeals[i])
                            mensaCardView.getLinLayoutDay().addView(meal1Layout)

                            val meal2Layout = MealLinearLayout(context)
                            meal2Layout.setMeal(dayMeals[i + 1])
                            meal2Layout.disableDivider()
                            mensaCardView.getLinLayoutDay().addView(meal2Layout)

                            break@loop
                        }
                    }

                } else {
                    val noFood = TextView(context)
                    noFood.text = resources.getString(R.string.mensa_closed)
                    noFood.setTextColor(ContextCompat.getColor(context!!, R.color.textPrimary))
                    noFood.textSize = 18.0F
                    noFood.setTypeface(null, Typeface.BOLD)
                    noFood.textAlignment = View.TEXT_ALIGNMENT_CENTER
                    noFood.setPadding(7,7,7,7)

                    mensaCardView.getLinLayoutDay().addView(noFood)
                }

                linLayout_Home.addView(mensaCardView)
            }
        }

    }

    /**
     * add the current timetable to the home screen
     */
    private fun addTimeTable() {
        val dayIndex = NotRetardedCalendar().getDayOfWeekIndex()
        var helpLesson = LessonLinearLayout(context!!)
        val dayCardView = DayCardView(context!!)
        dayCardView.setDayHeading(resources.getString(R.string.today_date, formatter.format(Calendar.getInstance().time)))

        doAsync {
            uiThread {

                if (timetables[0].days.isNotEmpty() && dayIndex < 6) {

                    // for all timeslots of the day
                    for ((tsIndex, timeslot) in timetables[0].days[dayIndex].timeslots.withIndex()) {

                        for(lesson in timeslot) {
                            if(lesson.lessonSubject.isNotEmpty()) {

                                val lessonLayout = LessonLinearLayout(context!!)
                                lessonLayout.setLesson(lesson, DataTypes().getTime()[tsIndex])
                                dayCardView.getLinLayoutDay().addView(lessonLayout)

                                if (lesson != timeslot.last()) {
                                    lessonLayout.disableDivider()
                                }

                                helpLesson = lessonLayout
                            }
                        }
                    }

                    helpLesson.disableDivider()
                    // if there are no lessons don't show the dayCardView
                    if (dayCardView.getLinLayoutDay().childCount > 1) {
                        linLayout_Home.addView(dayCardView)
                    } else {
                        // TODO we could display the next day with a lecture
                        val noLesson = DayCardView(context!!)
                        noLesson.setDayHeading(resources.getString(R.string.no_lesson_today))

                        linLayout_Home.addView(noLesson)
                    }
                } else {
                    if (dayIndex == 6) {
                        // if that's the case it's sunday
                        val noLesson = DayCardView(context!!)
                        noLesson.setDayHeading(resources.getString(R.string.no_lesson_today))

                        linLayout_Home.addView(noLesson)
                    } else {
                        MaterialDialog(context!!)
                            .title(R.string.error)
                            .message(R.string.gen_tt_error)
                            .show()
                        // TODO log the error and send feedback
                    }

                }

            }
        }

    }

}
